import json
from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import requests

from ... import errors
from ...models.get_conversations_order import GetConversationsOrder
from ...models.get_conversations_order_by import GetConversationsOrderBy
from ...models.get_conversations_response_200 import GetConversationsResponse200
from ...models.get_conversations_response_400 import GetConversationsResponse400
from ...models.get_conversations_response_401 import GetConversationsResponse401
from ...models.get_conversations_response_404 import GetConversationsResponse404
from ...models.get_conversations_response_500 import GetConversationsResponse500
from ...models.get_conversations_user_filter import GetConversationsUserFilter
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: int,
    *,
    client: {},
    page: Union[Unset, None, int] = 1,
    order: Union[Unset, None, GetConversationsOrder] = GetConversationsOrder.DESC,
    order_by: Union[Unset, None, GetConversationsOrderBy] = GetConversationsOrderBy.ID,
    user_filter: Union[Unset, None, GetConversationsUserFilter] = GetConversationsUserFilter.ALL,
    name: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/projects/{projectId}/conversations".format(client.base_url, projectId=project_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["page"] = page

    json_order: Union[Unset, None, str] = UNSET
    if not isinstance(order, Unset):
        json_order = order if order else None

    params["order"] = json_order

    json_order_by: Union[Unset, None, str] = UNSET
    if not isinstance(order_by, Unset):
        json_order_by = order_by if order_by else None

    params["orderBy"] = json_order_by

    json_user_filter: Union[Unset, None, str] = UNSET
    if not isinstance(user_filter, Unset):
        json_user_filter = user_filter if user_filter else None

    params["userFilter"] = json_user_filter

    params["name"] = name

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "allow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(*, client: {}, response: None) -> Optional[
    Union[
        GetConversationsResponse200,
        GetConversationsResponse400,
        GetConversationsResponse401,
        GetConversationsResponse404,
        GetConversationsResponse500,
    ]
]:
    if response.status_code == HTTPStatus.OK:
        response_200 = GetConversationsResponse200.from_dict(json.loads(response.text))

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = GetConversationsResponse400.from_dict(json.loads(response.text))

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = GetConversationsResponse401.from_dict(json.loads(response.text))

        return response_401
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = GetConversationsResponse404.from_dict(json.loads(response.text))

        return response_404
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = GetConversationsResponse500.from_dict(json.loads(response.text))

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: {}, response: None, content: Optional[bytes] = None) -> Response[
    Union[
        GetConversationsResponse200,
        GetConversationsResponse400,
        GetConversationsResponse401,
        GetConversationsResponse404,
        GetConversationsResponse500,
    ]
]:
    parse = _parse_response(client=client, response=response)
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content if content is None else content,
        headers=response.headers,
        parsed=parse,
    )


def sync_detailed(
    project_id: int,
    *,
    client: {},
    page: Union[Unset, None, int] = 1,
    order: Union[Unset, None, GetConversationsOrder] = GetConversationsOrder.DESC,
    order_by: Union[Unset, None, GetConversationsOrderBy] = GetConversationsOrderBy.ID,
    user_filter: Union[Unset, None, GetConversationsUserFilter] = GetConversationsUserFilter.ALL,
    name: Union[Unset, None, str] = UNSET,
):
    """List all conversations for a project.

     Retrieve all conversations associated with a project based on its unique projectId. This endpoint
    allows you to fetch a collection of conversations related to a specific project.

    Args:
        project_id (int):  Example: 1.
        page (Union[Unset, None, int]):  Default: 1.
        order (Union[Unset, None, GetConversationsOrder]):  Default: GetConversationsOrder.DESC.
            Example: desc.
        order_by (Union[Unset, None, GetConversationsOrderBy]):  Default:
            GetConversationsOrderBy.ID. Example: id.
        user_filter (Union[Unset, None, GetConversationsUserFilter]):  Default:
            GetConversationsUserFilter.ALL. Example: all.
        name (Union[Unset, None, str]):  Example: Ask me anything.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetConversationsResponse200, GetConversationsResponse400, GetConversationsResponse401, GetConversationsResponse404, GetConversationsResponse500]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        client=client,
        page=page,
        order=order,
        order_by=order_by,
        user_filter=user_filter,
        name=name,
    )

    response = requests.request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_id: int,
    *,
    client: {},
    page: Union[Unset, None, int] = 1,
    order: Union[Unset, None, GetConversationsOrder] = GetConversationsOrder.DESC,
    order_by: Union[Unset, None, GetConversationsOrderBy] = GetConversationsOrderBy.ID,
    user_filter: Union[Unset, None, GetConversationsUserFilter] = GetConversationsUserFilter.ALL,
    name: Union[Unset, None, str] = UNSET,
) -> Optional[
    Union[
        GetConversationsResponse200,
        GetConversationsResponse400,
        GetConversationsResponse401,
        GetConversationsResponse404,
        GetConversationsResponse500,
    ]
]:
    """List all conversations for a project.

     Retrieve all conversations associated with a project based on its unique projectId. This endpoint
    allows you to fetch a collection of conversations related to a specific project.

    Args:
        project_id (int):  Example: 1.
        page (Union[Unset, None, int]):  Default: 1.
        order (Union[Unset, None, GetConversationsOrder]):  Default: GetConversationsOrder.DESC.
            Example: desc.
        order_by (Union[Unset, None, GetConversationsOrderBy]):  Default:
            GetConversationsOrderBy.ID. Example: id.
        user_filter (Union[Unset, None, GetConversationsUserFilter]):  Default:
            GetConversationsUserFilter.ALL. Example: all.
        name (Union[Unset, None, str]):  Example: Ask me anything.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetConversationsResponse200, GetConversationsResponse400, GetConversationsResponse401, GetConversationsResponse404, GetConversationsResponse500]
    """

    return sync_detailed(
        project_id=project_id,
        client=client,
        page=page,
        order=order,
        order_by=order_by,
        user_filter=user_filter,
        name=name,
    ).parsed


async def asyncio_detailed(
    project_id: int,
    *,
    client: {},
    page: Union[Unset, None, int] = 1,
    order: Union[Unset, None, GetConversationsOrder] = GetConversationsOrder.DESC,
    order_by: Union[Unset, None, GetConversationsOrderBy] = GetConversationsOrderBy.ID,
    user_filter: Union[Unset, None, GetConversationsUserFilter] = GetConversationsUserFilter.ALL,
    name: Union[Unset, None, str] = UNSET,
) -> Response[
    Union[
        GetConversationsResponse200,
        GetConversationsResponse400,
        GetConversationsResponse401,
        GetConversationsResponse404,
        GetConversationsResponse500,
    ]
]:

    kwargs = _get_kwargs(
        project_id=project_id,
        client=client,
        page=page,
        order=order,
        order_by=order_by,
        user_filter=user_filter,
        name=name,
    )

    response = requests.request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: int,
    *,
    client: {},
    page: Union[Unset, None, int] = 1,
    order: Union[Unset, None, GetConversationsOrder] = GetConversationsOrder.DESC,
    order_by: Union[Unset, None, GetConversationsOrderBy] = GetConversationsOrderBy.ID,
    user_filter: Union[Unset, None, GetConversationsUserFilter] = GetConversationsUserFilter.ALL,
    name: Union[Unset, None, str] = UNSET,
) -> Optional[
    Union[
        GetConversationsResponse200,
        GetConversationsResponse400,
        GetConversationsResponse401,
        GetConversationsResponse404,
        GetConversationsResponse500,
    ]
]:
    """List all conversations for a project.

     Retrieve all conversations associated with a project based on its unique projectId. This endpoint
    allows you to fetch a collection of conversations related to a specific project.

    Args:
        project_id (int):  Example: 1.
        page (Union[Unset, None, int]):  Default: 1.
        order (Union[Unset, None, GetConversationsOrder]):  Default: GetConversationsOrder.DESC.
            Example: desc.
        order_by (Union[Unset, None, GetConversationsOrderBy]):  Default:
            GetConversationsOrderBy.ID. Example: id.
        user_filter (Union[Unset, None, GetConversationsUserFilter]):  Default:
            GetConversationsUserFilter.ALL. Example: all.
        name (Union[Unset, None, str]):  Example: Ask me anything.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetConversationsResponse200, GetConversationsResponse400, GetConversationsResponse401, GetConversationsResponse404, GetConversationsResponse500]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            client=client,
            page=page,
            order=order,
            order_by=order_by,
            user_filter=user_filter,
            name=name,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateSourceResponse201DataSettings")


@attr.s(auto_attribs=True)
class CreateSourceResponse201DataSettings:
    """The project source settings

    Attributes:
        executive_js (Union[Unset, bool]): Whether the project source should execute JavaScript Default: True. Example:
            True.
        data_refresh_frequency (Union[Unset, CreateSourceResponse201DataSettingsDataRefreshFrequency]): The project
            source data refresh frequency Default: CreateSourceResponse201DataSettingsDataRefreshFrequency.NEVER. Example:
            never.
        create_new_pages (Union[Unset, bool]): Add new pages to project automatically during refresh project source
            Default: True. Example: True.
        remove_unexist_pages (Union[Unset, bool]): Remove pages from project automatically during refresh project source
            Default: True.
        refresh_existing_pages (Union[Unset, CreateSourceResponse201DataSettingsRefreshExistingPages]): Refresh existing
            page during refresh project source Default: CreateSourceResponse201DataSettingsRefreshExistingPages.NEVER.
            Example: never.
        sitemap_path (Union[Unset, str]): The project source sitemap path Example: https://example.com/sitemap.xml.
    """

    executive_js: Union[Unset, bool] = True
    data_refresh_frequency: Union[Unset, str] = UNSET
    create_new_pages: Union[Unset, bool] = True
    remove_unexist_pages: Union[Unset, bool] = True
    refresh_existing_pages: Union[Unset, str] = UNSET
    sitemap_path: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        executive_js = True if self.executive_js else False

        data_refresh_frequency: Union[Unset, str] = UNSET
        if not isinstance(self.data_refresh_frequency, Unset):
            data_refresh_frequency = self.data_refresh_frequency

        create_new_pages = True if self.create_new_pages else False

        remove_unexist_pages = True if self.remove_unexist_pages else False

        refresh_existing_pages: Union[Unset, str] = UNSET
        if not isinstance(self.refresh_existing_pages, Unset):
            refresh_existing_pages = self.refresh_existing_pages

        sitemap_path = self.sitemap_path

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if executive_js is not UNSET:
            field_dict["executive_js"] = executive_js
        if data_refresh_frequency is not UNSET:
            field_dict["data_refresh_frequency"] = data_refresh_frequency
        if create_new_pages is not UNSET:
            field_dict["create_new_pages"] = create_new_pages
        if remove_unexist_pages is not UNSET:
            field_dict["remove_unexist_pages"] = remove_unexist_pages
        if refresh_existing_pages is not UNSET:
            field_dict["refresh_existing_pages"] = refresh_existing_pages
        if sitemap_path is not UNSET:
            field_dict["sitemap_path"] = sitemap_path

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        executive_js = src_dict.get("executive_js")

        data_refresh_frequency = src_dict.get("data_refresh_frequency")

        create_new_pages = src_dict.get("create_new_pages")

        remove_unexist_pages = src_dict.get("remove_unexist_pages")

        refresh_existing_pages = src_dict.get("refresh_existing_pages")

        sitemap_path = src_dict.get("sitemap_path")

        create_source_response_201_data_settings = cls(
            executive_js=executive_js,
            data_refresh_frequency=data_refresh_frequency,
            create_new_pages=create_new_pages,
            remove_unexist_pages=remove_unexist_pages,
            refresh_existing_pages=refresh_existing_pages,
            sitemap_path=sitemap_path,
        )

        create_source_response_201_data_settings.additional_properties = src_dict
        return create_source_response_201_data_settings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CreateSourceResponse201DataPagesItemCrawlStatus(str, Enum):
    FAILED = "failed"
    LIMITED = "limited"
    NA = "n/a"
    OK = "ok"
    QUEUED = "queued"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UpdateProjectSettingsMultipartDataCitationsViewType(str, Enum):
    HIDE = "hide"
    SHOW = "show"
    USER = "user"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class SynchronizeSourceResponse403DataMessage(str, Enum):
    NEXT_INSTANT_SYNC_WILL_BE_AVAILABLE_IN_SECONDS_SECONDS = (
        "Next instant sync will be available in \{seconds\} seconds"
    )
    PROJECT_SOURCE_CANT_BE_SYNCED = "Project source can't be synced"
    YOUR_PLAN_NOT_ELIGIBLE_FOR_INSTANT_SYNC = "Your plan not eligible for instant sync"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetMetadataResponse200Data")


@attr.s(auto_attribs=True)
class GetMetadataResponse200Data:
    """
    Attributes:
        id (Union[Unset, int]): The unique identifier of the page Example: 1.
        url (Union[Unset, None, str]): The URL of the page Example: https://www.example.com.
        title (Union[Unset, None, str]): The title of the page Example: Example Domain.
        description (Union[Unset, None, str]): The description of the page Example: This domain is for use in
            illustrative examples in documents. You may use this domain in literature without prior coordination or asking
            for permission..
        image (Union[Unset, None, str]): The image of the page Example: https://www.example.com/image.png.
    """

    id: Union[Unset, int] = UNSET
    url: Union[Unset, None, str] = UNSET
    title: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    image: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        url = self.url
        title = self.title
        description = self.description
        image = self.image

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if url is not UNSET:
            field_dict["url"] = url
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if image is not UNSET:
            field_dict["image"] = image

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        id = src_dict.get("id")

        url = src_dict.get("url")

        title = src_dict.get("title")

        description = src_dict.get("description")

        image = src_dict.get("image")

        get_metadata_response_200_data = cls(
            id=id,
            url=url,
            title=title,
            description=description,
            image=image,
        )

        get_metadata_response_200_data.additional_properties = src_dict
        return get_metadata_response_200_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.queries_reports_response_200_data_query_status_item import (
        QueriesReportsResponse200DataQueryStatusItem,
    )


T = TypeVar("T", bound="QueriesReportsResponse200Data")


@attr.s(auto_attribs=True)
class QueriesReportsResponse200Data:
    """
    Attributes:
        total (Union[Unset, int]): Total number of queries over all conversations Example: 10.
        query_status (Union[Unset, List['QueriesReportsResponse200DataQueryStatusItem']]): Number of successful and
            failed queries over all conversations
    """

    total: Union[Unset, int] = UNSET
    query_status: Union[Unset, List["QueriesReportsResponse200DataQueryStatusItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        total = self.total
        query_status: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.query_status, Unset):
            query_status = []
            for query_status_item_data in self.query_status:
                query_status_item = query_status_item_data.to_dict()

                query_status.append(query_status_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if total is not UNSET:
            field_dict["total"] = total
        if query_status is not UNSET:
            for index, field_value in enumerate(query_status):
                field_dict[f"query_status[{index}]"] = field_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.queries_reports_response_200_data_query_status_item import (
            QueriesReportsResponse200DataQueryStatusItem,
        )

        total = src_dict.get("total")

        query_status = []
        _query_status = src_dict.get("query_status")
        for query_status_item_data in _query_status or []:
            query_status_item = QueriesReportsResponse200DataQueryStatusItem.from_dict(query_status_item_data)

            query_status.append(query_status_item)

        queries_reports_response_200_data = cls(
            total=total,
            query_status=query_status,
        )

        queries_reports_response_200_data.additional_properties = src_dict
        return queries_reports_response_200_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SynchronizeSourceResponse404DataMessage(str, Enum):
    PROJECT_SOURCE_ID_IS_REQUIRED = "Project source id is required"
    PROJECT_SOURCE_WITH_ID_SOURCEID_NOT_FOUND = "Project source with id {sourceId} not found"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="QueriesReportsResponse200DataQueryStatusItem")


@attr.s(auto_attribs=True)
class QueriesReportsResponse200DataQueryStatusItem:
    """
    Attributes:
        status (Union[Unset, QueriesReportsResponse200DataQueryStatusItemStatus]):
        count (Union[Unset, int]):  Example: 2.
    """

    status: Union[Unset, str] = UNSET
    count: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        count = self.count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status
        if count is not UNSET:
            field_dict["count"] = count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        status = src_dict.get("status")

        count = src_dict.get("count")

        queries_reports_response_200_data_query_status_item = cls(
            status=status,
            count=count,
        )

        queries_reports_response_200_data_query_status_item.additional_properties = src_dict
        return queries_reports_response_200_data_query_status_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.get_plugin_response_401_data import GetPluginResponse401Data


T = TypeVar("T", bound="GetPluginResponse401")


@attr.s(auto_attribs=True)
class GetPluginResponse401:
    """
    Attributes:
        status (Union[Unset, GetPluginResponse401Status]): The status of the response Example: error.
        url (Union[Unset, str]): The URL of the request Example: https://app.customgpt.ai/api/v1/projects/1.
        data (Union[Unset, GetPluginResponse401Data]):
    """

    status: Union[Unset, str] = UNSET
    url: Union[Unset, str] = UNSET
    data: Union[Unset, "GetPluginResponse401Data"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        url = self.url
        data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data, Unset):
            data = self.data.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status
        if url is not UNSET:
            field_dict["url"] = url
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.get_plugin_response_401_data import GetPluginResponse401Data

        status = src_dict.get("status")

        url = src_dict.get("url")

        _data = src_dict.get("data")
        data: Union[Unset, GetPluginResponse401Data]
        if isinstance(_data, Unset):
            data = UNSET
        else:
            data = GetPluginResponse401Data.from_dict(_data)

        get_plugin_response_401 = cls(
            status=status,
            url=url,
            data=data,
        )

        get_plugin_response_401.additional_properties = src_dict
        return get_plugin_response_401

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="MessagesConversationResponse500Data")


@attr.s(auto_attribs=True)
class MessagesConversationResponse500Data:
    """
    Attributes:
        code (Union[Unset, MessagesConversationResponse500DataCode]): The error status code Example: 500.
        message (Union[Unset, str]):  Example: Internal Server Error.
    """

    code: Union[Unset, str] = UNSET
    message: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code: Union[Unset, int] = UNSET
        if not isinstance(self.code, Unset):
            code = self.code

        message = self.message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if message is not UNSET:
            field_dict["message"] = message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        code = src_dict.get("code")

        message = src_dict.get("message")

        messages_conversation_response_500_data = cls(
            code=code,
            message=message,
        )

        messages_conversation_response_500_data.additional_properties = src_dict
        return messages_conversation_response_500_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import requests
from sseclient import SSEClient

from ... import errors
from ...models.send_message_cache_control import SendMessageCacheControl
from ...models.send_message_json_body import SendMessageJsonBody
from ...models.send_message_response_200 import SendMessageResponse200
from ...models.send_message_response_400 import SendMessageResponse400
from ...models.send_message_response_401 import SendMessageResponse401
from ...models.send_message_response_404 import SendMessageResponse404
from ...models.send_message_response_429 import SendMessageResponse429
from ...models.send_message_response_500 import SendMessageResponse500
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: int,
    session_id: str,
    *,
    client: {},
    json_body: SendMessageJsonBody,
    stream: Union[Unset, None, bool] = False,
    lang: Union[Unset, None, str] = "en",
    external_id: Union[Unset, None, str] = UNSET,
    cache_control: Union[Unset, None, SendMessageCacheControl] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/projects/{projectId}/conversations/{sessionId}/messages".format(
        client.base_url, projectId=project_id, sessionId=session_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()
    if not isinstance(cache_control, Unset) and cache_control is not None:
        headers["Cache-Control"] = str(cache_control)

    params: Dict[str, Any] = {}
    True if stream else False

    params["stream"] = 1 if stream else 0

    params["lang"] = lang

    params["external_id"] = external_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    if stream:
        headers["Accept"] = "text/event-stream"

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "allow_redirects": client.follow_redirects,
        "json": json_json_body,
        "params": params,
        "stream": stream,
    }


def _parse_response(*, client: {}, response: None) -> Optional[
    Union[
        SendMessageResponse200,
        SendMessageResponse400,
        SendMessageResponse401,
        SendMessageResponse404,
        SendMessageResponse429,
        SendMessageResponse500,
    ]
]:
    if response.status_code == HTTPStatus.OK:
        response_200 = SendMessageResponse200.from_dict(json.loads(response.text))

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = SendMessageResponse400.from_dict(json.loads(response.text))

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = SendMessageResponse401.from_dict(json.loads(response.text))

        return response_401
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = SendMessageResponse404.from_dict(json.loads(response.text))

        return response_404
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        response_429 = SendMessageResponse429.from_dict(json.loads(response.text))

        return response_429
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = SendMessageResponse500.from_dict(json.loads(response.text))

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: {}, response: None, content: Optional[bytes] = None) -> Response[
    Union[
        SendMessageResponse200,
        SendMessageResponse400,
        SendMessageResponse401,
        SendMessageResponse404,
        SendMessageResponse429,
        SendMessageResponse500,
    ]
]:
    parse = _parse_response(client=client, response=response)
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content if content is None else content,
        headers=response.headers,
        parsed=parse,
    )


def sync_detailed(
    project_id: int,
    session_id: str,
    *,
    client: {},
    json_body: SendMessageJsonBody,
    stream: Union[Unset, None, bool] = False,
    lang: Union[Unset, None, str] = "en",
    external_id: Union[Unset, None, str] = UNSET,
    cache_control: Union[Unset, None, SendMessageCacheControl] = UNSET,
):
    """Send a message to a conversation.

     Send a message to a conversation within a project identified by its unique projectId and sessionId.
    This endpoint enables you to send a new message to a specific conversation, facilitating seamless
    communication and collaboration within the project. By providing the projectId and sessionId, you
    can target the desired conversation and contribute to the ongoing discussion. This API endpoint
    supports real-time streaming, allowing for instant message delivery and dynamic updates which
    enables efficient and interactive communication between the user and chatbot.
    Here is an example to send a message to a conversation: [API](https://github.com/Poll-The-
    People/customgpt-
    cookbook/blob/main/examples/Create_a_new_conversation_and_send_a_message_to_the_conversation.ipynb)
    [SDK](https://github.com/Poll-The-People/customgpt-cookbook/blob/main/examples/SDK_Create_a_new_conv
    ersation_and_send_a_message_to_the_conversation.ipynb).

    Args:
        project_id (int):  Example: 1.
        session_id (str):  Example: 1.
        stream (Union[Unset, None, bool]):
        lang (Union[Unset, None, str]):  Default: 'en'.
        external_id (Union[Unset, None, str]):
        cache_control (Union[Unset, None, SendMessageCacheControl]):
        json_body (SendMessageJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[SendMessageResponse200, SendMessageResponse400, SendMessageResponse401, SendMessageResponse404, SendMessageResponse429, SendMessageResponse500]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        session_id=session_id,
        client=client,
        json_body=json_body,
        stream=stream,
        lang=lang,
        external_id=external_id,
        cache_control=cache_control,
    )

    response = requests.request(
        **kwargs,
    )

    if stream:
        return SSEClient(response)
    else:
        return _build_response(client=client, response=response)


def sync(
    project_id: int,
    session_id: str,
    *,
    client: {},
    json_body: SendMessageJsonBody,
    stream: Union[Unset, None, bool] = False,
    lang: Union[Unset, None, str] = "en",
    external_id: Union[Unset, None, str] = UNSET,
    cache_control: Union[Unset, None, SendMessageCacheControl] = UNSET,
) -> Optional[
    Union[
        SendMessageResponse200,
        SendMessageResponse400,
        SendMessageResponse401,
        SendMessageResponse404,
        SendMessageResponse429,
        SendMessageResponse500,
    ]
]:
    """Send a message to a conversation.

     Send a message to a conversation within a project identified by its unique projectId and sessionId.
    This endpoint enables you to send a new message to a specific conversation, facilitating seamless
    communication and collaboration within the project. By providing the projectId and sessionId, you
    can target the desired conversation and contribute to the ongoing discussion. This API endpoint
    supports real-time streaming, allowing for instant message delivery and dynamic updates which
    enables efficient and interactive communication between the user and chatbot.
    Here is an example to send a message to a conversation: [API](https://github.com/Poll-The-
    People/customgpt-
    cookbook/blob/main/examples/Create_a_new_conversation_and_send_a_message_to_the_conversation.ipynb)
    [SDK](https://github.com/Poll-The-People/customgpt-cookbook/blob/main/examples/SDK_Create_a_new_conv
    ersation_and_send_a_message_to_the_conversation.ipynb).

    Args:
        project_id (int):  Example: 1.
        session_id (str):  Example: 1.
        stream (Union[Unset, None, bool]):
        lang (Union[Unset, None, str]):  Default: 'en'.
        external_id (Union[Unset, None, str]):
        cache_control (Union[Unset, None, SendMessageCacheControl]):
        json_body (SendMessageJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[SendMessageResponse200, SendMessageResponse400, SendMessageResponse401, SendMessageResponse404, SendMessageResponse429, SendMessageResponse500]
    """

    return sync_detailed(
        project_id=project_id,
        session_id=session_id,
        client=client,
        json_body=json_body,
        stream=stream,
        lang=lang,
        external_id=external_id,
        cache_control=cache_control,
    ).parsed


async def asyncio_detailed(
    project_id: int,
    session_id: str,
    *,
    client: {},
    json_body: SendMessageJsonBody,
    stream: Union[Unset, None, bool] = False,
    lang: Union[Unset, None, str] = "en",
    external_id: Union[Unset, None, str] = UNSET,
    cache_control: Union[Unset, None, SendMessageCacheControl] = UNSET,
) -> Response[
    Union[
        SendMessageResponse200,
        SendMessageResponse400,
        SendMessageResponse401,
        SendMessageResponse404,
        SendMessageResponse429,
        SendMessageResponse500,
    ]
]:
    kwargs = _get_kwargs(
        project_id=project_id,
        session_id=session_id,
        client=client,
        json_body=json_body,
        stream=stream,
        lang=lang,
        external_id=external_id,
        cache_control=cache_control,
    )

    response = requests.request(
        **kwargs,
    )

    if stream:
        return SSEClient(response)
    else:
        return _build_response(client=client, response=response)


async def asyncio(
    project_id: int,
    session_id: str,
    *,
    client: {},
    json_body: SendMessageJsonBody,
    stream: Union[Unset, None, bool] = False,
    lang: Union[Unset, None, str] = "en",
    external_id: Union[Unset, None, str] = UNSET,
    cache_control: Union[Unset, None, SendMessageCacheControl] = UNSET,
) -> Optional[
    Union[
        SendMessageResponse200,
        SendMessageResponse400,
        SendMessageResponse401,
        SendMessageResponse404,
        SendMessageResponse429,
        SendMessageResponse500,
    ]
]:
    """Send a message to a conversation.

     Send a message to a conversation within a project identified by its unique projectId and sessionId.
    This endpoint enables you to send a new message to a specific conversation, facilitating seamless
    communication and collaboration within the project. By providing the projectId and sessionId, you
    can target the desired conversation and contribute to the ongoing discussion. This API endpoint
    supports real-time streaming, allowing for instant message delivery and dynamic updates which
    enables efficient and interactive communication between the user and chatbot.
    Here is an example to send a message to a conversation: [API](https://github.com/Poll-The-
    People/customgpt-
    cookbook/blob/main/examples/Create_a_new_conversation_and_send_a_message_to_the_conversation.ipynb)
    [SDK](https://github.com/Poll-The-People/customgpt-cookbook/blob/main/examples/SDK_Create_a_new_conv
    ersation_and_send_a_message_to_the_conversation.ipynb).

    Args:
        project_id (int):  Example: 1.
        session_id (str):  Example: 1.
        stream (Union[Unset, None, bool]):
        lang (Union[Unset, None, str]):  Default: 'en'.
        external_id (Union[Unset, None, str]):
        cache_control (Union[Unset, None, SendMessageCacheControl]):
        json_body (SendMessageJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[SendMessageResponse200, SendMessageResponse400, SendMessageResponse401, SendMessageResponse404, SendMessageResponse429, SendMessageResponse500]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            session_id=session_id,
            client=client,
            json_body=json_body,
            stream=stream,
            lang=lang,
            external_id=external_id,
            cache_control=cache_control,
        )
    ).parsed



from enum import Enum


class SynchronizeSourceResponse201DataSettingsDataRefreshFrequency(str, Enum):
    ADVANCED = "advanced"
    DAILY = "daily"
    MONTHLY = "monthly"
    NEVER = "never"
    WEEKLY = "weekly"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="MessagesConversationResponse200DataConversation")


@attr.s(auto_attribs=True)
class MessagesConversationResponse200DataConversation:
    """
    Attributes:
        created_at (Union[Unset, datetime.datetime]): When was this conversation created? Example: 2023-04-30 16:43:53.
        updated_at (Union[Unset, datetime.datetime]): When was this conversation updated? Example: 2023-04-30 16:43:53.
        id (Union[Unset, int]): Conversation ID Example: 1.
        name (Union[Unset, str]): Conversation name Example: Conversation 1.
        project_id (Union[Unset, str]): Project ID for this conversation Example: 1.
        created_by (Union[Unset, str]): User ID for the user who created this conversation Example: 1.
        session_id (Union[Unset, str]): Session ID for this conversation Example: f1b9aaf0-5e4e-11eb-ae93-0242ac130002.
    """

    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    id: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    project_id: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    session_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        id = self.id
        name = self.name
        project_id = self.project_id
        created_by = self.created_by
        session_id = self.session_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if session_id is not UNSET:
            field_dict["session_id"] = session_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        _created_at = src_dict.get("created_at")
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = src_dict.get("updated_at")
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        id = src_dict.get("id")

        name = src_dict.get("name")

        project_id = src_dict.get("project_id")

        created_by = src_dict.get("created_by")

        session_id = src_dict.get("session_id")

        messages_conversation_response_200_data_conversation = cls(
            created_at=created_at,
            updated_at=updated_at,
            id=id,
            name=name,
            project_id=project_id,
            created_by=created_by,
            session_id=session_id,
        )

        messages_conversation_response_200_data_conversation.additional_properties = src_dict
        return messages_conversation_response_200_data_conversation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetConversationsUserFilter(str, Enum):
    ALL = "all"
    ANONYMOUS = "anonymous"
    TEAM_MEMBER = "team_member"

    def __str__(self) -> str:
        return str(self.value)



from enum import IntEnum


class ReindexPageResponse400DataCode(IntEnum):
    VALUE_400 = 400
    VALUE_401 = 401
    VALUE_403 = 403
    VALUE_404 = 404
    VALUE_500 = 500
    VALUE_503 = 503

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UpdateSourceJsonBodyRefreshExistingPages(str, Enum):
    ALWAYS = "always"
    IF_UPDATED = "if_updated"
    NEVER = "never"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateSourceJsonBodyRefreshScheduleItem")


@attr.s(auto_attribs=True)
class UpdateSourceJsonBodyRefreshScheduleItem:
    """
    Attributes:
        days (Union[Unset, List[int]]): Index of days in which sitemap should be refreshed. Starts from 0 (Sunday) to 6
            (Saturday). Example: [0, 1, 4, 6].
        hours (Union[Unset, List[str]]): List of times in which sitemap should be refreshed. Must be in HH:MM format.
            Example: ['00:00', '08:00', '23:45'].
    """

    days: Union[Unset, List[int]] = UNSET
    hours: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        days: Union[Unset, List[int]] = UNSET
        if not isinstance(self.days, Unset):
            days = self.days

        hours: Union[Unset, List[str]] = UNSET
        if not isinstance(self.hours, Unset):
            hours = self.hours

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if days is not UNSET:
            for index, field_value in enumerate(days):
                field_dict[f"days[{index}]"] = field_value
        if hours is not UNSET:
            for index, field_value in enumerate(hours):
                field_dict[f"hours[{index}]"] = field_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        days = cast(List[int], src_dict.get("days"))

        hours = cast(List[str], src_dict.get("hours"))

        update_source_json_body_refresh_schedule_item = cls(
            days=days,
            hours=hours,
        )

        update_source_json_body_refresh_schedule_item.additional_properties = src_dict
        return update_source_json_body_refresh_schedule_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="StatsProjectResponse200Data")


@attr.s(auto_attribs=True)
class StatsProjectResponse200Data:
    """
    Attributes:
        pages_found (Union[Unset, int]): Number of pages found for the project Example: 100.
        pages_crawled (Union[Unset, int]): Number of pages crawled for the project Example: 100.
        pages_indexed (Union[Unset, int]): Number of pages indexed for the project Example: 100.
        crawl_credits_used (Union[Unset, int]): Number of crawl credits used for the project Example: 100.
        query_credits_used (Union[Unset, int]): Number of query credits used for the project Example: 100.
        total_queries (Union[Unset, int]): Number of queries used for the project Example: 100.
        total_words_indexed (Union[Unset, int]): Number of index credits used for the project Example: 100.
        total_storage_credits_used (Union[Unset, int]): Number of words stored for the project Example: 100.
    """

    pages_found: Union[Unset, int] = UNSET
    pages_crawled: Union[Unset, int] = UNSET
    pages_indexed: Union[Unset, int] = UNSET
    crawl_credits_used: Union[Unset, int] = UNSET
    query_credits_used: Union[Unset, int] = UNSET
    total_queries: Union[Unset, int] = UNSET
    total_words_indexed: Union[Unset, int] = UNSET
    total_storage_credits_used: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        pages_found = self.pages_found
        pages_crawled = self.pages_crawled
        pages_indexed = self.pages_indexed
        crawl_credits_used = self.crawl_credits_used
        query_credits_used = self.query_credits_used
        total_queries = self.total_queries
        total_words_indexed = self.total_words_indexed
        total_storage_credits_used = self.total_storage_credits_used

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if pages_found is not UNSET:
            field_dict["pages_found"] = pages_found
        if pages_crawled is not UNSET:
            field_dict["pages_crawled"] = pages_crawled
        if pages_indexed is not UNSET:
            field_dict["pages_indexed"] = pages_indexed
        if crawl_credits_used is not UNSET:
            field_dict["crawl_credits_used"] = crawl_credits_used
        if query_credits_used is not UNSET:
            field_dict["query_credits_used"] = query_credits_used
        if total_queries is not UNSET:
            field_dict["total_queries"] = total_queries
        if total_words_indexed is not UNSET:
            field_dict["total_words_indexed"] = total_words_indexed
        if total_storage_credits_used is not UNSET:
            field_dict["total_storage_credits_used"] = total_storage_credits_used

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        pages_found = src_dict.get("pages_found")

        pages_crawled = src_dict.get("pages_crawled")

        pages_indexed = src_dict.get("pages_indexed")

        crawl_credits_used = src_dict.get("crawl_credits_used")

        query_credits_used = src_dict.get("query_credits_used")

        total_queries = src_dict.get("total_queries")

        total_words_indexed = src_dict.get("total_words_indexed")

        total_storage_credits_used = src_dict.get("total_storage_credits_used")

        stats_project_response_200_data = cls(
            pages_found=pages_found,
            pages_crawled=pages_crawled,
            pages_indexed=pages_indexed,
            crawl_credits_used=crawl_credits_used,
            query_credits_used=query_credits_used,
            total_queries=total_queries,
            total_words_indexed=total_words_indexed,
            total_storage_credits_used=total_storage_credits_used,
        )

        stats_project_response_200_data.additional_properties = src_dict
        return stats_project_response_200_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.prompt_history_metadata import PromptHistoryMetadata


T = TypeVar("T", bound="PromptHistory")


@attr.s(auto_attribs=True)
class PromptHistory:
    """
    Attributes:
        id (Union[Unset, int]): The unique identifier of the prompt history. Example: 1.
        user_id (Union[Unset, int]): The unique identifier of the user. Example: 1.
        user_query (Union[Unset, str]): The user prompt query. Example: What is the meaning of life?.
        openai_response (Union[Unset, str]): The OpenAI response to the user prompt query. Example: The meaning of life
            is to be happy..
        created_at (Union[Unset, datetime.datetime]): The date and time the prompt history was created. Example:
            2021-01-01 00:00:00.
        updated_at (Union[Unset, datetime.datetime]): The date and time the prompt history was last updated. Example:
            2021-01-01 00:00:00.
        conversation_id (Union[Unset, int]): The unique identifier of the conversation. Example: 1.
        citations (Union[Unset, List[int]]): The citations for the prompt history. Example: [1, 2, 3].
        metadata (Union[Unset, PromptHistoryMetadata]):  Example: {'user_ip': '127.0.0.1', 'user_agent': 'Mozilla/5.0
            (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko)', 'external_id': 'ext_id_1234567890',
            'request_source': 'web'}.
    """

    id: Union[Unset, int] = UNSET
    user_id: Union[Unset, int] = UNSET
    user_query: Union[Unset, str] = UNSET
    openai_response: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    conversation_id: Union[Unset, int] = UNSET
    citations: Union[Unset, List[int]] = UNSET
    metadata: Union[Unset, "PromptHistoryMetadata"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        user_id = self.user_id
        user_query = self.user_query
        openai_response = self.openai_response
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        conversation_id = self.conversation_id
        citations: Union[Unset, List[int]] = UNSET
        if not isinstance(self.citations, Unset):
            citations = self.citations

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if user_id is not UNSET:
            field_dict["user_id"] = user_id
        if user_query is not UNSET:
            field_dict["user_query"] = user_query
        if openai_response is not UNSET:
            field_dict["openai_response"] = openai_response
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if conversation_id is not UNSET:
            field_dict["conversation_id"] = conversation_id
        if citations is not UNSET:
            for index, field_value in enumerate(citations):
                field_dict[f"citations[{index}]"] = field_value
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.prompt_history_metadata import PromptHistoryMetadata

        id = src_dict.get("id")

        user_id = src_dict.get("user_id")

        user_query = src_dict.get("user_query")

        openai_response = src_dict.get("openai_response")

        _created_at = src_dict.get("created_at")
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = src_dict.get("updated_at")
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        conversation_id = src_dict.get("conversation_id")

        citations = cast(List[int], src_dict.get("citations"))

        _metadata = src_dict.get("metadata")
        metadata: Union[Unset, PromptHistoryMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = PromptHistoryMetadata.from_dict(_metadata)

        prompt_history = cls(
            id=id,
            user_id=user_id,
            user_query=user_query,
            openai_response=openai_response,
            created_at=created_at,
            updated_at=updated_at,
            conversation_id=conversation_id,
            citations=citations,
            metadata=metadata,
        )

        prompt_history.additional_properties = src_dict
        return prompt_history

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class UpdateMetadataResponse404DataMessage(str, Enum):
    PAGE_ID_IS_REQUIRED = "Page id is required"
    PAGE_WITH_ID_PAGEID_NOT_FOUND = "Page with id {pageId} not found"

    def __str__(self) -> str:
        return str(self.value)
